@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,7 +19,7 @@
 
 from __future__ import print_function
 
-import os, sys, subprocess, tempfile, atexit, shutil, re
+import os, sys, subprocess, tempfile, atexit, shutil, re, ast
 
 # Make sure we flush after every print, the "-u" option does more than that
 # and this is easy enough.
@@ -30,8 +30,8 @@ def my_print(*args, **kwargs):
 
 
 def check_output(*popenargs, **kwargs):
-    if 'stdout' in kwargs:
-        raise ValueError('stdout argument not allowed, it will be overridden.')
+    if "stdout" in kwargs:
+        raise ValueError("stdout argument not allowed, it will be overridden.")
 
     process = subprocess.Popen(
         stdout = subprocess.PIPE,
@@ -50,8 +50,8 @@ def check_output(*popenargs, **kwargs):
     return output
 
 def check_result(*popenargs, **kwargs):
-    if 'stdout' in kwargs:
-        raise ValueError('stdout argument not allowed, it will be overridden.')
+    if "stdout" in kwargs:
+        raise ValueError("stdout argument not allowed, it will be overridden.")
 
     process = subprocess.Popen(
         stdout = subprocess.PIPE,
@@ -98,7 +98,7 @@ def setup(needs_io_encoding = False, silent = False):
             """\
 import sys, os;\
 print(".".join(str(s) for s in list(sys.version_info)[:3]));\
-print(("x86_64" if "AMD64" in sys.version else "x86") if os.name=="nt" else os.uname()[4]);\
+print(("x86_64" if "AMD64" in sys.version else "x86") if os.name == "nt" else os.uname()[4]);\
 """,
         ),
         stderr = subprocess.STDOUT
@@ -108,7 +108,7 @@ print(("x86_64" if "AMD64" in sys.version else "x86") if os.name=="nt" else os.u
     python_version = version_output.split(b"\n")[0].strip()
     python_arch = version_output.split(b"\n")[1].strip()
 
-    if sys.version.startswith("3"):
+    if sys.version.startswith('3'):
         python_arch = python_arch.decode()
         python_version = python_version.decode()
 
@@ -143,7 +143,7 @@ def getTempDir():
                 os.path.dirname(
                     os.path.abspath(sys.modules[ "__main__" ].__file__)
                 )
-            ) + "-",
+            ) + '-',
             dir    = tempfile.gettempdir() if
                          not os.path.exists("/var/tmp") else
                     "/var/tmp"
@@ -160,7 +160,7 @@ def getTempDir():
     return tmp_dir
 
 
-def convertUsing2to3(path):
+def convertUsing2to3(path, force = False):
     command = [
         os.environ["PYTHON"],
         "-m",
@@ -168,9 +168,12 @@ def convertUsing2to3(path):
         path
     ]
 
-    with open(os.devnull, "w") as stderr:
-        if check_result(command, stderr = stderr):
-            return path, False
+    if not force:
+        with open(path) as source_file:
+            if "xrange" not in source_file.read():
+                with open(os.devnull, 'w') as stderr:
+                    if check_result(command, stderr = stderr):
+                        return path, False
 
     filename = os.path.basename(path)
 
@@ -198,7 +201,7 @@ def convertUsing2to3(path):
         new_path
     ]
 
-    with open(os.devnull, "w") as devnull:
+    with open(os.devnull, 'w') as devnull:
         check_output(
             command,
             stderr = devnull
@@ -219,7 +222,7 @@ def decideFilenameVersionSkip(filename):
     if filename.endswith("27.py") and python_version.startswith("2.6"):
         return False
 
-    if filename.endswith("_2.py") and python_version.startswith("3"):
+    if filename.endswith("_2.py") and python_version.startswith('3'):
         return False
 
     # Skip tests that require Python 3.2 at least.
@@ -234,6 +237,10 @@ def decideFilenameVersionSkip(filename):
     if filename.endswith("34.py") and python_version < "3.4":
         return False
 
+    # Skip tests that require Python 3.5 at least.
+    if filename.endswith("35.py") and python_version < "3.5":
+        return False
+
     return True
 
 
@@ -256,7 +263,8 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
         "silent"
     ]
 
-    command += extra_flags
+    if extra_flags is not None:
+        command += extra_flags
 
     command += search_mode.getExtraFlags(dirname, filename)
 
@@ -303,6 +311,14 @@ def hasDebugPython():
     if os.path.exists(debug_python):
         return True
 
+    # On Windows systems, these work.
+    debug_python = os.environ["PYTHON"]
+    if debug_python.lower().endswith(".exe"):
+        debug_python = debug_python[:-4]
+    debug_python = debug_python + "_d.exe"
+    if os.path.exists(debug_python):
+        return True
+
     # For other Python, if it's the one also executing the runner, which is
     # very probably the case, we check that. We don't check the provided
     # binary here, this could be done as well.
@@ -343,23 +359,56 @@ def getDependsExePath():
 
     return depends_exe
 
+def isExecutableCommand(command):
+    path = os.environ["PATH"]
 
-def getRuntimeTraceOfLoadedFiles(path,trace_error=True):
+    suffixes = (".exe",) if os.name == "nt" else ("",)
+
+    for part in path.split(os.pathsep):
+        if not part:
+            continue
+
+        for suffix in suffixes:
+            if os.path.isfile(os.path.join(part, command + suffix)):
+                return True
+
+    return False
+
+
+def getRuntimeTraceOfLoadedFiles(path, trace_error = True):
     """ Returns the files loaded when executing a binary. """
 
     result = []
 
     if os.name == "posix":
-        if sys.platform == "darwin":
+        if sys.platform == "darwin" or \
+           sys.platform.startswith("freebsd"):
+            if not isExecutableCommand("dtruss"):
+                sys.exit(
+                    """\
+Error, needs 'dtruss' on your system to scan used libraries."""
+                )
+
+            if not isExecutableCommand("sudo"):
+                sys.exit(
+                    """\
+Error, needs 'sudo' on your system to scan used libraries."""
+                )
+
             args = (
                 "sudo",
                 "dtruss",
-                "-f",
                 "-t",
                 "open",
                 path
             )
         else:
+            if not isExecutableCommand("strace"):
+                sys.exit(
+                    """\
+Error, needs 'strace' on your system to scan used libraries."""
+                )
+
             args = (
                 "strace",
                 "-e", "file",
@@ -373,7 +422,7 @@ def getRuntimeTraceOfLoadedFiles(path,trace_error=True):
             stderr = subprocess.PIPE
         )
 
-        stdout_strace, stderr_strace = process.communicate()
+        _stdout_strace, stderr_strace = process.communicate()
 
         open(path+".strace","wb").write(stderr_strace)
 
@@ -416,7 +465,7 @@ def getRuntimeTraceOfLoadedFiles(path,trace_error=True):
                 re.findall(b'"(.*?)(?:\\\\0)?"', line)
             )
 
-        if sys.version.startswith("3"):
+        if sys.version.startswith('3'):
             result = [s.decode("utf-8") for s in result]
     elif os.name == "nt":
         subprocess.call(
@@ -445,14 +494,14 @@ def getRuntimeTraceOfLoadedFiles(path,trace_error=True):
             if "| Module List |" in line:
                 break
 
-            if "]" not in line:
+            if ']' not in line:
                 continue
 
             # Skip missing DLLs, apparently not needed anyway.
-            if "?" in line[:line.find("]")]:
+            if '?' in line[:line.find(']')]:
                 continue
 
-            dll_filename = line[line.find("]")+2:-1]
+            dll_filename = line[line.find(']')+2:-1]
             assert os.path.isfile(dll_filename), dll_filename
 
             # The executable itself is of course exempted.
@@ -478,7 +527,7 @@ def hasModule(module_name):
             "-c"
             "import %s" % module_name
         ),
-        stdout = open(os.devnull,"w"),
+        stdout = open(os.devnull,'w'),
         stderr = subprocess.STDOUT
     )
 
@@ -524,7 +573,7 @@ def checkReferenceCount(checked_function, max_rounds = 10):
     assert max_rounds > 0
     for count in range(max_rounds):
         gc.collect()
-        ref_count1 = sys.gettotalrefcount()
+        ref_count1 = sys.gettotalrefcount()  # @UndefinedVariable
 
         if explain and count == max_rounds - 1:
             snapObjRefCntMap(True)
@@ -539,7 +588,7 @@ def checkReferenceCount(checked_function, max_rounds = 10):
         if explain and count == max_rounds - 1:
             snapObjRefCntMap(False)
 
-        ref_count2 = sys.gettotalrefcount()
+        ref_count2 = sys.gettotalrefcount()  # @UndefinedVariable
 
         if ref_count1 == ref_count2:
             result = True
@@ -557,10 +606,10 @@ def checkReferenceCount(checked_function, max_rounds = 10):
 
             for key in m1.keys():
                 if key not in m2:
-                    print("*" * 80)
+                    print('*' * 80)
                     print(key)
                 elif m1[key] != m2[key]:
-                    print("*" * 80)
+                    print('*' * 80)
                     print(key)
                 else:
                     pass
@@ -575,9 +624,9 @@ def checkReferenceCount(checked_function, max_rounds = 10):
 
 
 def createSearchMode():
-    search_mode = len( sys.argv ) > 1 and sys.argv[1] == "search"
-    start_at = sys.argv[2] if len( sys.argv ) > 2 else None
-    coverage_mode = len( sys.argv ) > 1 and sys.argv[1] == "coverage"
+    search_mode = len(sys.argv) > 1 and sys.argv[1] == "search"
+    start_at = sys.argv[2] if len(sys.argv) > 2 else None
+    coverage_mode = len(sys.argv) > 1 and sys.argv[1] == "coverage"
 
     class SearchModeBase:
         def __init__(self):
@@ -615,8 +664,10 @@ def createSearchMode():
                 dirname,
                 filename,
                 filename.replace(".py", ""),
+                filename.split(".")[0],
                 path,
-                path.replace(".py", "")
+                path.replace(".py", ""),
+
             )
 
         def isCoverage(self):
@@ -632,10 +683,10 @@ def createSearchMode():
 
         return SearchModeCoverage()
     elif search_mode and start_at:
-        start_at = start_at.replace("/", os.path.sep)
+        start_at = start_at.replace('/', os.path.sep)
 
         class SearchModeByPattern(SearchModeBase):
-            def __init__( self ):
+            def __init__(self):
                 SearchModeBase.__init__(self)
 
                 self.active = False
@@ -662,7 +713,10 @@ def createSearchMode():
         return SearchModeImmediate()
 
 def reportSkip(reason, dirname, filename):
-    my_print("Skipped, %s (%s)." % (os.path.join(dirname, filename), reason))
+    case = os.path.join(dirname, filename)
+    case = os.path.normpath(case)
+
+    my_print("Skipped, %s (%s)." % (case, reason))
 
 def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr):
     import gc
@@ -731,3 +785,190 @@ def withPythonPathChange(python_path):
             del os.environ["PYTHONPATH"]
         else:
             os.environ["PYTHONPATH"] = old_path
+
+@contextmanager
+def withExtendedExtraOptions(*args):
+    assert args
+    old_value = os.environ.get("NUITKA_EXTRA_OPTIONS", None)
+
+    value = old_value
+
+    for arg in args:
+        if value is None:
+            value = arg
+        else:
+            value += ' ' + arg
+
+    os.environ[ "NUITKA_EXTRA_OPTIONS" ] = value
+
+    yield
+
+    if old_value is None:
+        del os.environ[ "NUITKA_EXTRA_OPTIONS" ]
+    else:
+        os.environ[ "NUITKA_EXTRA_OPTIONS" ] = old_value
+
+
+def indentedCode(codes, count):
+    """ Indent code, used for generating test codes.
+
+    """
+    return '\n'.join( ' ' * count + line if line else "" for line in codes )
+
+
+def convertToPython(doctests, line_filter = None):
+    """ Convert give doctest string to static Python code.
+
+    """
+    import doctest
+    code = doctest.script_from_examples(doctests)
+
+    if code.endswith('\n'):
+        code += "#\n"
+    else:
+        assert False
+
+    output = []
+    inside = False
+
+    def getPrintPrefixed(evaluated):
+        try:
+            node = ast.parse(evaluated.lstrip(), "eval")
+        except SyntaxError:
+            return evaluated
+
+        if node.body[0].__class__.__name__ == "Expr":
+            count = 0
+
+            while evaluated.startswith(' ' * count):
+                count += 1
+
+            if sys.version_info < (3,):
+                modified = (count-1) * ' ' + "print " + evaluated
+                return (count-1) * ' ' + ("print 'Line %d'" % line_number) + "\n" + modified
+            else:
+                modified = (count-1) * ' ' + "print(" + evaluated + "\n)\n"
+                return (count-1) * ' ' + ("print('Line %d'" % line_number) + ")\n" + modified
+        else:
+            return evaluated
+
+    def getTried(evaluated):
+        if sys.version_info < (3,):
+            return """
+try:
+%(evaluated)s
+except Exception as __e:
+    print "Occurred", type(__e), __e
+""" % { "evaluated" : indentedCode(getPrintPrefixed(evaluated).split('\n'), 4) }
+        else:
+            return """
+try:
+%(evaluated)s
+except Exception as __e:
+    print("Occurred", type(__e), __e)
+""" % { "evaluated" : indentedCode(getPrintPrefixed(evaluated).split('\n'), 4) }
+
+    def isOpener(evaluated):
+        evaluated = evaluated.lstrip()
+
+        if evaluated == "":
+            return False
+
+        if evaluated.split()[0] in ("def", "class", "for", "while", "try:", "except", "except:", "finally:", "else:"):
+            return True
+        else:
+            return False
+
+    for line_number, line in enumerate(code.split('\n')):
+        # print "->", inside, line
+
+        if line_filter is not None and line_filter(line):
+            continue
+
+        if inside and len(line) > 0 and line[0].isalnum() and not isOpener(line):
+            output.append(getTried('\n'.join(chunk)))  # @UndefinedVariable
+
+            chunk = []
+            inside = False
+
+        if inside and not (line.startswith('#') and line.find("SyntaxError:") != -1):
+            chunk.append(line)
+        elif line.startswith('#'):
+            if line.find("SyntaxError:") != -1:
+                # print "Syntax error detected"
+
+                if inside:
+                    # print "Dropping chunk", chunk
+
+                    chunk = []
+                    inside = False
+                else:
+                    del output[-1]
+        elif isOpener(line):
+            inside = True
+            chunk = [line]
+        elif line.strip() == "":
+            output.append(line)
+        else:
+            output.append(getTried(line))
+
+    return '\n'.join(output).rstrip() + '\n'
+
+
+def compileLibraryPath(search_mode, path, stage_dir, decide, action):
+    my_print("Checking standard library path:", path)
+    global active
+
+    for root, dirnames, filenames in os.walk(path):
+        dirnames_to_remove = [
+            dirname
+            for dirname in dirnames
+            if "-" in dirname
+        ]
+
+        for dirname in dirnames_to_remove:
+            dirnames.remove(dirname)
+
+        dirnames.sort()
+
+        filenames = [
+            filename
+            for filename in filenames
+            if decide(root, filename)
+        ]
+
+        for filename in sorted(filenames):
+            if not search_mode.consider(root, filename):
+                continue
+
+            full_path = os.path.join(root, filename)
+
+            my_print(full_path, ':', end = ' ')
+            sys.stdout.flush()
+
+            action(stage_dir, path, full_path)
+
+
+def compileLibraryTest(search_mode, stage_dir, decide, action):
+    my_dirname = os.path.dirname(__file__)
+
+    paths = [
+        path
+        for path in
+        sys.path
+        if not path.startswith(my_dirname)
+    ]
+
+    my_print("Using standard library paths:")
+    for path in paths:
+        my_print(path)
+
+
+    for path in paths:
+        compileLibraryPath(
+            search_mode = search_mode,
+            path        = path,
+            stage_dir   = stage_dir,
+            decide      = decide,
+            action      = action
+        )
